 /*
  * Khoros: $Id: dscale.c,v 1.2 1992/03/20 23:34:45 dkhoros Exp $
  */

#if !defined(lint) && !defined(SABER)
static char rcsid[] = "Khoros: $Id: dscale.c,v 1.2 1992/03/20 23:34:45 dkhoros Exp $";
#endif

 /*
  * $Log: dscale.c,v $
 * Revision 1.2  1992/03/20  23:34:45  dkhoros
 * VirtualPatch5
 *
  */

/*
 *----------------------------------------------------------------------
 *
 * Copyright 1991, University of New Mexico.  All rights reserved.
 * Permission to copy and modify this software and its documen-
 * tation only for internal use in your organization is hereby
 * granted, provided that this notice is retained thereon and
 * on all copies.  UNM makes no representations as to the sui-
 * tability and operability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 * 
 * UNM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FIT-
 * NESS.  IN NO EVENT SHALL UNM BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY OTHER DAMAGES WHAT-
 * SOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PER-
 * FORMANCE OF THIS SOFTWARE.
 * 
 * No other rights, including, for example, the right to redis-
 * tribute this software and its documentation or the right to
 * prepare derivative works, are granted unless specifically
 * provided in a separate license agreement.
 *---------------------------------------------------------------------
 */

#include "unmcopyright.h"        /* Copyright 1991 by UNM */

/*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 >>>>
 >>>>         File Name: dscale.c
 >>>>
 >>>>      Program Name: dscale
 >>>>
 >>>>      Program Desc: 
 >>>>	
 >>>>	1D Rescale Data
 >>>>	
 >>>>	

 >>>>            Author: Ramiro Jordan, Jeremy Worley
 >>>>
 >>>> Date Last Updated: Sat Feb 15 16:36:00 1992
 >>>>
 >>>>          Routines: main- the main program for dscale
 >>>> 		 gw_usage - gives usage of the program
 >>>> 		 gw_args  - gets arguments of program from command line
 >>>>
 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<*/


#include "dscale.h"


/****************************************************************
 *
 * Routine Name:  main program for dscale
 *
 *       Input:  
 *        -i  input data file (xviff file) 
 *        -o  resulting output file (xviff file) 
 *
 *
 *        [-s] specify the scaling parameter 
 *        [-d] data processing direction (0 = down vectors, 1 = across bands)
 *
 *
 *
 ****************************************************************/


main(argc, argv)
int argc;
char *argv[];
{

/* -main_variable_list */
struct xvimage *image, *readimage();
int    process_dir;
float  scale;
/* -main_variable_list_end */

	program = VStrcpy(argv[0]);
	ac = argc;
	av = argv;

	khoros_init();

	gw_get_args();

/* -main_before_lib_call */
if (check_args()) exit(1);

READIMAGE(image);
CHECKTYPE(program,image);
CHECKINPUT(program,image);

scale = dscale->s_float;
process_dir = dscale->d_logic;
/* -main_before_lib_call_end */

/* -main_library_call */
if(! ldscale ( image, scale, process_dir ) )
        {
            (void) fprintf(stderr, "ldscale Failed\n");
            exit(1);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
    writeimage(dscale->o_file,image);
/* -main_after_lib_call_end */


	khoros_close();
}


/****************************************************************
*
* Routine Name:  gw_usage 
*
*
* Purpose:  prints out the usage for dscale 
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_usage()
{
	fprintf(stderr, "dscale :\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t1D Rescale Data\n");
	fprintf(stderr, "\t\n");
	fprintf(stderr, "\t\n");

/* -usage_additions */
(void) fprintf(stderr,"where \"d\" direction of processing:\n");
(void) fprintf(stderr,"\t\"d=0\" down vectors \n");
(void) fprintf(stderr,"\t\"d=1\" across bands \n");
/* -usage_additions_end */

	fprintf(stderr,"\t-i  input data file (xviff file)  (infile)\n");
	fprintf(stderr,"\t-o  resulting output file (xviff file)  (outfile)\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\n");
	fprintf(stderr,"\t[-s]  specify the scaling parameter (float) [0.1]\n");
	fprintf(stderr,"\t[-d]  data processing direction (0 = down vectors, 1 = across bands) (boolean) [true]\n");
	fprintf(stderr, " \n");
	fprintf(stderr, "\n");
	fprintf(stderr, "\t[-V] Gives the version for dscale\n");
	fprintf(stderr, "\t[-U] Gives the usage for dscale\n");
	fprintf(stderr, "\t[-P] Prompts for command line options\n");
	fprintf(stderr, "\t[-A [file1]] Creates the answer file called dscale.ans or file1 \n");
	fprintf(stderr, "\t[-a [file1]] Uses dscale.ans or file1 as the answer file \n\n\n");

}


/****************************************************************
*
* Routine Name:  gw_getargs 
*
*
* Purpose:  gets arguments off command line for dscale
*
* Input:  none 
*
* Output:  none 
*
* Written By:  automatically generated by ghostwriter
*
****************************************************************/


gw_get_args()
{

char	*i_pstr = 
	"\nEnter: (i) input data file (xviff file) \n        {infile}: "; 
char	*o_pstr = 
	"\nEnter: (o) resulting output file (xviff file) \n        {outfile}: "; 
char	*s_pstr = 
	"\nEnter: (s) specify the scaling parameter \n        {float [0.100000] }: "; 
char	*d_pstr = 
	"\nEnter: (d) data processing direction (0 = down vectors, 1 = across bands) \n        {boolean, (y/n)  [yes] }: "; 
char  answer_file[512], temp[512];



int	fid = -1,
	error, query, ok, count;
	int prompt_flag = FALSE;

	/*
	 * print version if necessary
	 */
	vgparml(fid, "-V", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    printf ("\n%s: %s: Version %d.%d\n\n",
	    "dscale", RELEASE_NAME, RELEASE_MAJOR, RELEASE_MINOR);
	    exit(1);
	}

	/*
	 * print usage if necessary
	 */
	vgparml(fid, "-U", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	{
	    gw_usage();
	    exit(1);
	}

	/*
	 * see if interactive prompting is desired
	 */
	vgparml(fid, "-P", "noprompt", &query, 0, 0, &error);
	if (error == 0)
	    prompt_flag = TRUE;

	/*
	 * see if answer file is to be used
	 */
	vgparms(fid, "-a", "noprompt", answer_file, 0, 0, &error);
	if (error == 0)
	{
	     if (VStrlen(answer_file) == 0)
		sprintf(answer_file, "dscale.ans");
	     while (fid < 0)
	     {
		if ((fid = open(answer_file, O_RDONLY, 0666))== -1)
		{
		    fprintf(stderr, "dscale: can't open answer file \"%s\"",
			    answer_file); 
		    fprintf(stderr, "please re-enter: ");
		    (void) gets(answer_file);
		    if (VStrlen(answer_file) == 0)
			sprintf(answer_file, "dscale.ans");
		}

	     }

	}

	/*
	 * allocate the dscale structure
	 */
	dscale = (dscale_struct *) 
	       calloc((unsigned)1, (unsigned) sizeof (dscale_struct));

	/*
	 * get required arguments for dscale
	 */
	if (prompt_flag == TRUE)
	    fprintf(stderr,"\nRequired Arguments:\n\n");

	if (!(dscale->i_flag = vget_infile(fid, prompt_flag,
			"-i", i_pstr, &(dscale->i_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(dscale->i_flag))
		{
		    fprintf(stderr, "\t\ti is required, please re-enter: \n");
		    dscale->i_flag = vget_infile(fid, prompt_flag,
		    "-i", i_pstr, &(dscale->i_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-i' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	if (!(dscale->o_flag = vget_outfile(fid, prompt_flag,
			"-o", o_pstr, &(dscale->o_file))
))
	{
	    if (prompt_flag == TRUE)
	    {
		while(!(dscale->o_flag))
		{
		    fprintf(stderr, "\t\to is required, please re-enter: \n");
		    dscale->o_flag = vget_outfile(fid, prompt_flag,
		    "-o", o_pstr, &(dscale->o_file))
;

		}
	    }
	    else
	    {
		fprintf(stderr, "Error: '-o' is a required argument\n");
		gw_usage();
		exit(1);
	    }
	}

	/*
	 * get optional arguments for dscale
	 */
	if (prompt_flag == TRUE)
	   fprintf(stderr,"\nOptional Arguments:\n\n");

	dscale->s_flag = vget_float(fid, prompt_flag,
	"-s", s_pstr, &(dscale->s_float),
	0.100000, 0.000000, 0.000000);

	dscale->d_flag = vget_logic(fid, prompt_flag,
	"-d", d_pstr, &(dscale->d_logic), 1);


}



